#include <util/XMLUniDefs.hpp>
#include <sax2/Attributes.hpp>
#include <stdlib.h>
#include "wmEnotesConfig.h"
#include "SAX2Handlers.h"

NoteStruct *note;
char* buffer;

// ---------------------------------------------------------------------------
//  Local const data
//
//  Note: This is the 'safe' way to do these strings. If you compiler supports
//        L"" style strings, and portability is not a concern, you can use
//        those types constants directly.
// ---------------------------------------------------------------------------
XERCES_CPP_NAMESPACE_BEGIN
static const XMLCh  gEndElement[] = { XERCES_CPP_NAMESPACE_QUALIFIER chOpenAngle, chForwardSlash, chNull };
static const XMLCh  gEndPI[] = { chQuestion, chCloseAngle, chNull };
static const XMLCh  gStartPI[] = { chOpenAngle, chQuestion, chNull };
static const XMLCh  gXMLDecl1[] =
{
        chOpenAngle, chQuestion, chLatin_x, chLatin_m, chLatin_l
    ,   chSpace, chLatin_v, chLatin_e, chLatin_r, chLatin_s, chLatin_i
    ,   chLatin_o, chLatin_n, chEqual, chDoubleQuote, chDigit_1, chPeriod
    ,   chDigit_0, chDoubleQuote, chSpace, chLatin_e, chLatin_n, chLatin_c
    ,   chLatin_o, chLatin_d, chLatin_i, chLatin_n, chLatin_g, chEqual
    ,   chDoubleQuote, chNull
};

static const XMLCh  gXMLDecl2[] =
{
        chDoubleQuote, chQuestion, chCloseAngle
    ,   chCR, chLF, chNull
};


// ---------------------------------------------------------------------------
//  SAX2Handlers: Constructors and Destructor
// ---------------------------------------------------------------------------
SAX2Handlers::SAX2Handlers( const   char* const              encodingName
                                    , const XERCES_CPP_NAMESPACE_QUALIFIER XMLFormatter::UnRepFlags unRepFlags
									, const bool					 expandNamespaces) :

    fFormatter
    (
        encodingName
        , this
        , XERCES_CPP_NAMESPACE_QUALIFIER XMLFormatter::NoEscapes
        , unRepFlags
    ),
	fExpandNS ( expandNamespaces )
{
    //
    //  Go ahead and output an XML Decl with our known encoding. This
    //  is not the best answer, but its the best we can do until we
    //  have SAX2 support.
    //
    fFormatter << XERCES_CPP_NAMESPACE_QUALIFIER gXMLDecl1 << fFormatter.getEncodingName() << XERCES_CPP_NAMESPACE_QUALIFIER gXMLDecl2;
}

SAX2Handlers::~SAX2Handlers()
{
}


// ---------------------------------------------------------------------------
//  SAX2Handlers: Overrides of the output formatter target interface
// ---------------------------------------------------------------------------
void SAX2Handlers::writeChars(const XMLByte* const toWrite)
{
}

void SAX2Handlers::writeChars(const XMLByte* const toWrite,
                                   const unsigned int count,
                                   XERCES_CPP_NAMESPACE_QUALIFIER XMLFormatter* const formatter)
{
    // For this one, just dump them to the standard output
    // Surprisingly, Solaris was the only platform on which
    // required the char* cast to print out the string correctly.
    // Without the cast, it was printing the pointer value in hex.
    // Quite annoying, considering every other platform printed
    // the string with the explicit cast to char* below.
	//cout.write((char *) toWrite, (int) count);
	//cout.flush();
}


// ---------------------------------------------------------------------------
//  SAX2Handlers: Overrides of the SAX ErrorHandler interface
// ---------------------------------------------------------------------------
void SAX2Handlers::error(const XERCES_CPP_NAMESPACE_QUALIFIER SAXParseException& e)
{
    printf("Error at file %s, line %d, char %d \nmessage:%s\n",XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(e.getSystemId()),e.getLineNumber(),e.getColumnNumber(),XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(e.getMessage()));
}

void SAX2Handlers::fatalError(const XERCES_CPP_NAMESPACE_QUALIFIER SAXParseException& e)
{
    printf("Fatal error at file %s, line %d, char %d \nmessage:%s\n",XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(e.getSystemId()),e.getLineNumber(),e.getColumnNumber(),XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(e.getMessage()));
}

void SAX2Handlers::warning(const XERCES_CPP_NAMESPACE_QUALIFIER SAXParseException& e)
{
    printf("Warning at file %s, line %d, char %d \nmessage:%s\n",XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(e.getSystemId()),e.getLineNumber(),e.getColumnNumber(),XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(e.getMessage()));
}


// ---------------------------------------------------------------------------
//  SAX2Handlers: Overrides of the SAX DTDHandler interface
// ---------------------------------------------------------------------------
void SAX2Handlers::unparsedEntityDecl(const     XMLCh* const name
                                          , const   XMLCh* const publicId
                                          , const   XMLCh* const systemId
                                          , const   XMLCh* const notationName)
{
    // Not used at this time
}


void SAX2Handlers::notationDecl(const   XMLCh* const name
                                    , const XMLCh* const publicId
                                    , const XMLCh* const systemId)
{
    // Not used at this time
}


// ---------------------------------------------------------------------------
//  SAX2Handlers: Overrides of the SAX DocumentHandler interface
// ---------------------------------------------------------------------------
void SAX2Handlers::characters(const     XMLCh* const    chars
                                  , const   unsigned int    length)
{
   // fFormatter.formatBuf(chars, length, XERCES_CPP_NAMESPACE_QUALIFIER XMLFormatter::CharEscapes);
	buffer = (char*)malloc(length+1);
	sprintf(buffer,XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(chars));
}


void SAX2Handlers::endDocument()
{
}


void SAX2Handlers::endElement(const XMLCh* const uri, 
								 const XMLCh* const localname, 
								 const XMLCh* const qname) {

    // No escapes are legal here
	if (XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(localname),(const char *)"alarm") == 0)
		sprintf(note->alarm,buffer);
	if (XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(localname),(const char *)"fontName") == 0)
		sprintf(note->fontName,buffer);
	if (XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(localname),(const char *)"textColor") == 0)
		sprintf(note->textColor,buffer);
	if (XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(localname),(const char *)"backgdColor") == 0)
		sprintf(note->backgdColor,buffer);
	if (XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(localname),(const char *)"title") == 0)
		sprintf(note->title,buffer);
	if (XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(localname),(const char *)"text") == 0)
		sprintf(note->text,buffer);

/*    fFormatter << XERCES_CPP_NAMESPACE_QUALIFIER XMLFormatter::NoEscapes << gEndElement ;
	if ( fExpandNS ) {
		if (XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString(uri,XMLUni::fgEmptyString) != 0)
				fFormatter  << uri << chColon;
		fFormatter << localname << chCloseAngle;
		}
	else
		fFormatter << qname << chCloseAngle;*/
}


void SAX2Handlers::ignorableWhitespace( const   XMLCh* const chars
                                            ,const  unsigned int length)
{
    fFormatter.formatBuf(chars, length, XERCES_CPP_NAMESPACE_QUALIFIER XMLFormatter::NoEscapes);
}


void SAX2Handlers::processingInstruction(const  XMLCh* const target
                                            , const XMLCh* const data)
{
    fFormatter << XERCES_CPP_NAMESPACE_QUALIFIER XMLFormatter::NoEscapes << XERCES_CPP_NAMESPACE_QUALIFIER gStartPI  << target;
    if (data)
        fFormatter << XERCES_CPP_NAMESPACE_QUALIFIER chSpace << data;
    fFormatter << XERCES_CPP_NAMESPACE_QUALIFIER XMLFormatter::NoEscapes << XERCES_CPP_NAMESPACE_QUALIFIER gEndPI;
}


void SAX2Handlers::startDocument() {
}


void SAX2Handlers::startElement(const   XMLCh* const    uri,
									const   XMLCh* const    localname,
									const   XMLCh* const    qname,
                                    const   XERCES_CPP_NAMESPACE_QUALIFIER Attributes&		attributes) {

    // The name has to be representable without any escapes
/*    fFormatter  << XMLFormatter::NoEscapes << chOpenAngle ;
	if ( fExpandNS ) {
		if (XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString(uri,XMLUni::fgEmptyString) != 0)
				fFormatter  << uri << chColon;
		fFormatter << localname ;
		}
	else
		fFormatter << qname ;*/

	if (XERCES_CPP_NAMESPACE_QUALIFIER  XMLString::compareIString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(localname),(const char *)"note") == 0) {
		note = (NoteStruct*)malloc(sizeof(NoteStruct));
		unsigned int len = attributes.getLength();
		for (unsigned int index = 0; index < len; index++) {
		if ( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getLocalName(index)),(const char *)"border") == 0) note->border = atoi(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getValue(index)));
		if ( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getLocalName(index)),(const char *)"angle") == 0) note->angle = atoi(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getValue(index)));
		if ( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getLocalName(index)),(const char *)"sticky") == 0) note->sticky = atoi(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getValue(index)));
		if ( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getLocalName(index)),(const char *)"ontop") == 0) note->ontop = atoi(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getValue(index)));
		if ( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getLocalName(index)),(const char *)"transparent") == 0) note->transparent = atoi(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getValue(index)));
		if ( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getLocalName(index)),(const char *)"shade") == 0) note->shade = atoi(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getValue(index)));
		if ( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getLocalName(index)),(const char *)"desktopBackgd") == 0) note->desktopBackgd = atoi(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getValue(index)));
		if ( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getLocalName(index)),(const char *)"paddingX") == 0) note->paddingX = atoi(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getValue(index)));
		if ( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getLocalName(index)),(const char *)"paddingY") == 0) note->paddingY = atoi( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getValue(index)));
		}
	}
        //
        //  Again the name has to be completely representable. But the
        //  attribute can have refs and requires the attribute style
        //  escaping.
        //
/*printf("localname=%s-value=%s\n", XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getLocalName(index)), XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(attributes.getValue(index)));
        fFormatter  << XMLFormatter::NoEscapes << chSpace ;
		if ( fExpandNS ) {
			if ( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareIString(attributes.getURI(index),XMLUni::fgEmptyString) != 0)
				fFormatter  << attributes.getURI(index) << chColon;
			fFormatter  << attributes.getLocalName(index) ;
			}
		else
			fFormatter  << attributes.getQName(index) ;

		fFormatter  << chEqual << chDoubleQuote
                    << XMLFormatter::AttrEscapes
		            << attributes.getValue(index)
                    << XMLFormatter::NoEscapes
                    << chDoubleQuote;
    }
    fFormatter << chCloseAngle;*/
}

void SAX2Handlers::getNote(NoteStruct **n) {
	*n = note;
}

void SAX2Handlers::setNote(NoteStruct *n) {
	note = n;
}

XERCES_CPP_NAMESPACE_END

