#include <util/PlatformUtils.hpp>
#include <util/TransService.hpp>
#include <sax2/SAX2XMLReader.hpp>
#include <sax2/XMLReaderFactory.hpp>
#include "wmEnotesConfig.h"
#include "SAX2Handlers.h"

#include <sys/stat.h>
#include "fancy.xpm"
#include "backgd.xpm"
#include "text.xpm"

NoteStruct *note_to_configure;
CONFIG_WINDOW config_window;
GtkWidget *mainwindow;
char *notes_temp_file = "~/temp.xml";
time_t config_time = 0;

/*
 * MAIN FUNCTION...
*/
int main(int argc, char **argv) {
	if (argc<2) usage();
	if (argc>2) notes_temp_file = argv[2];
	gtk_init(&argc,&argv);
	gtk_timeout_add(100,(timer_func),NULL);
	initialize_config_window();
	load_config(notes_temp_file);
	//gtk_widget_hide (window);
	gtk_main ();
	return(0);
}

void usage(void) {

	fprintf(stderr, "\nwmEnotesConfig - programming: Philippe Vigneron\n\n");
	fprintf(stderr, "usage:\n");
	fprintf(stderr, "-f <note temp file>  xml file where to store the temporary note info when configuring\n");
	fprintf(stderr, "\n");
	exit(0);
}
/*
 * EVENT LOOP FOR GTK...
*/
gint timer_func(gpointer p) {
struct stat buf;

	if (stat(notes_temp_file,&buf)!=-1) {
		if (config_time != buf.st_mtime) {
			config_time = buf.st_mtime;
			load_config(notes_temp_file);
			printf("file has been modified...\n");
			}
	}
	
	return(1);
}

int save_config(void) {
FILE *f;

	f = fopen(notes_temp_file,"w");

	if (f) {
		fprintf(f,"<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n\n");
		fprintf(f,"<ENotes>\n");
		fprintf(f,"\t<note border=\"%d\" angle=\"%d\" sticky=\"%d\" ontop=\"%d\" transparent=\"%d\" shade=\"%d\" desktopBackgd=\"%d\" paddingX=\"%d\" paddingY=\"%d\">\n",note_to_configure->border,note_to_configure->angle,note_to_configure->sticky,note_to_configure->ontop,note_to_configure->transparent,note_to_configure->shade,note_to_configure->desktopBackgd,note_to_configure->paddingX,note_to_configure->paddingY);
		fprintf(f,"\t\t<alarm>%s</alarm>\n",note_to_configure->alarm);
		fprintf(f,"\t\t<fontName>%s</fontName>\n",note_to_configure->fontName);
		fprintf(f,"\t\t<textColor>%s</textColor>\n",note_to_configure->textColor);
		fprintf(f,"\t\t<backgdColor>%s</backgdColor>\n",note_to_configure->backgdColor);
		fprintf(f,"\t\t<title>%s</title>\n",note_to_configure->title);
		fprintf(f,"\t\t<text>%s</text>\n",note_to_configure->text);
		fprintf(f,"\t</note>\n");
		fprintf(f,"</ENotes>\n");
		fclose(f);
		return 0;
		}
	else {
		printf("could not write the file [%s], note cannot be configured!\n",notes_temp_file);
		return -1;
		}
}

void load_config(char *filename) {
XERCES_CPP_NAMESPACE_QUALIFIER SAX2XMLReader* parser=NULL;

	note_to_configure = NULL;
	try {
		XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::Initialize();
		parser = XERCES_CPP_NAMESPACE_QUALIFIER XMLReaderFactory::createXMLReader();
		}

	catch (const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch) {
		printf("Error during initialization! %s\n",XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(toCatch.getMessage()));
		}

	try {
		SAX2Handlers handler("iso-8859-1", XERCES_CPP_NAMESPACE_QUALIFIER XMLFormatter::UnRep_CharRef,false);
		if (parser) {
			parser->setContentHandler(&handler);
			parser->setErrorHandler(&handler);
			parser->parse(filename);
			handler.getNote(&note_to_configure);
			}
		}

	catch (const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch) {
		printf("An error occured\n  Error:%s\n",XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(toCatch.getMessage()));
		}

	Set_Config_Fields();
}

/*
 * _____________________________________________________________________
 * _____________________________________________________________________
 * 
 * Config window stuff...
 * _____________________________________________________________________
 * _____________________________________________________________________
*/
void initialize_config_window(void){
GtkWidget *fancy_p;
GtkWidget *backgd_p;
GtkWidget *text_p;
GtkWidget *vbox;
GtkWidget *hbox1;
GtkWidget *hbox2;
GtkWidget *hbox3;
GtkWidget *apply_b;
GtkWidget *ok_b;
GtkWidget *cancel_b;

#ifdef DEBUG
printf("Initializing config window...\n");
#endif

	// create a new window 
	mainwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect (GTK_OBJECT (mainwindow), "delete_event", GTK_SIGNAL_FUNC (hide_config), NULL);
	gtk_signal_connect (GTK_OBJECT (mainwindow), "destroy", GTK_SIGNAL_FUNC (hide_config), NULL);

	gtk_quit_add_destroy(1,GTK_OBJECT (mainwindow));

	gtk_window_set_default_size(GTK_WINDOW(mainwindow), 500, 400);
	gtk_widget_show (mainwindow);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);
	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox1);
	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox2);
	hbox3 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox3);

	//title text field
	config_window.title_tf = gtk_entry_new_with_max_length(128);
	gtk_widget_show (config_window.title_tf);
	gtk_box_pack_start( GTK_BOX(vbox), config_window.title_tf, 0, 0, 5  );
	//gtk_widget_set_usize(title_tf, 70, 20);

	fancy_p = create_fancy_panel();
	backgd_p = create_backgd_panel();
	text_p = create_text_panel();

	//apply/ok/cancel buttons
	ok_b = gtk_button_new_with_label ("OK");
	gtk_widget_show (ok_b);
	gtk_signal_connect (GTK_OBJECT (ok_b), "clicked", GTK_SIGNAL_FUNC (flush_data), NULL);
	gtk_signal_connect (GTK_OBJECT (ok_b), "clicked", GTK_SIGNAL_FUNC (hide_config), NULL);

	apply_b = gtk_button_new_with_label ("Apply");
	gtk_widget_show (apply_b);
	gtk_signal_connect (GTK_OBJECT (apply_b), "clicked", GTK_SIGNAL_FUNC (flush_data), NULL);

	cancel_b = gtk_button_new_with_label ("Cancel");
	gtk_widget_show (cancel_b);
	gtk_signal_connect (GTK_OBJECT (cancel_b), "clicked", GTK_SIGNAL_FUNC (hide_config), NULL);

	gtk_box_pack_start( GTK_BOX(hbox3), apply_b, 0, 0, 5  );
	gtk_box_pack_start( GTK_BOX(hbox3), ok_b, 0, 0, 5  );
	gtk_box_pack_start( GTK_BOX(hbox3), cancel_b, 0, 0, 5  );
	gtk_box_pack_start( GTK_BOX(hbox1), fancy_p, 0, 0, 5 );
	gtk_box_pack_start( GTK_BOX(hbox1), backgd_p, 0, 0, 5  );
	gtk_box_pack_start( GTK_BOX(hbox2), text_p, 0, 0, 5  );
	gtk_box_pack_start( GTK_BOX(vbox), hbox1, 0, 0, 5  );
	gtk_box_pack_start( GTK_BOX(vbox), hbox2, 0, 0, 5  );
	gtk_box_pack_start( GTK_BOX(vbox), hbox3, 0, 0, 5  );
	gtk_container_add( GTK_CONTAINER(mainwindow), vbox );
}

//FANCY PANEL________________________________________________________________
GtkWidget* create_fancy_panel(void) {
GtkWidget *angle_sc_label;
GtkWidget *alarm_tf_label;
GtkWidget *back;
GdkBitmap *mask;
GdkPixmap *pixmap;
GtkStyle *style;
GtkWidget *panel;
GtkAdjustment *adj;

	panel = gtk_fixed_new();
	gtk_widget_show (panel);

	style = gtk_widget_get_style( mainwindow );
	pixmap = gdk_pixmap_create_from_xpm_d(mainwindow->window, &mask,&style->bg[GTK_STATE_NORMAL], fancy_xpm);
	back = gtk_pixmap_new (pixmap, mask);
	gtk_widget_set_usize(back, 200, 200);
	gtk_fixed_put(GTK_FIXED(panel),back, 0, 0);
	gtk_widget_show (back);

	//border
	config_window.border_cb = gtk_check_button_new_with_label("border?");
	gtk_widget_set_usize(config_window.border_cb, 175, 20);
	gtk_fixed_put(GTK_FIXED(panel),config_window.border_cb, 10, 10);
	gtk_widget_show (config_window.border_cb);

	//sticky
	config_window.sticky_cb = gtk_check_button_new_with_label("sticky?");
	gtk_widget_set_usize(config_window.sticky_cb, 175, 20);
	gtk_fixed_put(GTK_FIXED(panel),config_window.sticky_cb, 10, 30);
	gtk_widget_show (config_window.sticky_cb);

	//ontop
	config_window.ontop_cb = gtk_check_button_new_with_label("ontop?");
	gtk_widget_set_usize(config_window.ontop_cb, 175, 20);
	gtk_fixed_put(GTK_FIXED(panel),config_window.ontop_cb, 10, 50);
	gtk_widget_show (config_window.ontop_cb);

	//angle
	angle_sc_label = gtk_label_new("angle");
	gtk_fixed_put(GTK_FIXED(panel),angle_sc_label, 10, 85);
	gtk_widget_show (angle_sc_label);
	adj = (GtkAdjustment *) gtk_adjustment_new (0, 0.0, 360.0, 1.0, 5.0, 0.0);
	config_window.angle_sc = gtk_hscale_new (GTK_ADJUSTMENT (adj));
	gtk_widget_set_usize(config_window.angle_sc, 100, 30);
	gtk_range_set_update_policy (GTK_RANGE (config_window.angle_sc), GTK_UPDATE_CONTINUOUS);
	gtk_scale_set_digits (GTK_SCALE(config_window.angle_sc), 0);
	gtk_scale_set_value_pos (GTK_SCALE(config_window.angle_sc), GTK_POS_TOP);
	gtk_scale_set_draw_value (GTK_SCALE(config_window.angle_sc), TRUE);
	gtk_fixed_put(GTK_FIXED(panel),config_window.angle_sc, 50, 70);
	gtk_widget_show (config_window.angle_sc);

	//alarm text field
	alarm_tf_label = gtk_label_new("alarm");
	gtk_fixed_put(GTK_FIXED(panel),alarm_tf_label, 10, 115);
	gtk_widget_show (alarm_tf_label);
	config_window.alarm_tf = gtk_entry_new_with_max_length(128);
	gtk_widget_set_usize(config_window.alarm_tf, 125, 20);
	gtk_fixed_put(GTK_FIXED(panel),config_window.alarm_tf, 50, 110);
	gtk_widget_show (config_window.alarm_tf);

	return(panel);
}

//BACKGD PANEL________________________________________________________________
GtkWidget* create_backgd_panel(void) {
GtkWidget *shade_sc_label;
GtkWidget *backgdFiles_b;
GtkWidget *backgdColor_b;
GtkWidget *back;
GdkBitmap *mask;
GdkPixmap *pixmap;
GtkStyle *style;
GtkWidget *panel;
GtkAdjustment *adj;

	panel = gtk_fixed_new();
	gtk_widget_show (panel);

	style = gtk_widget_get_style( mainwindow );
	pixmap = gdk_pixmap_create_from_xpm_d(mainwindow->window, &mask, &style->bg[GTK_STATE_NORMAL],backgd_xpm);
	back = gtk_pixmap_new (pixmap, mask);
	gtk_widget_set_usize(back, 250, 200);
	gtk_fixed_put(GTK_FIXED(panel),back, 0, 0);
	gtk_widget_show (back);

	//backgd text field
	config_window.backgd_tf = gtk_entry_new_with_max_length(128);
	gtk_widget_set_usize(config_window.backgd_tf, 130, 20);
	gtk_fixed_put(GTK_FIXED(panel),config_window.backgd_tf, 70, 120);
	gtk_widget_show (config_window.backgd_tf);

	//transparency
	config_window.transparent_cb = gtk_check_button_new_with_label("transparency?");
	gtk_widget_set_usize(config_window.transparent_cb, 175, 20);
	gtk_fixed_put(GTK_FIXED(panel),config_window.transparent_cb, 10, 10);
	gtk_widget_show (config_window.transparent_cb);

	//desktop backgd
	config_window.desktopbg_cb = gtk_check_button_new_with_label("desktop?");
	gtk_widget_set_usize(config_window.desktopbg_cb, 175, 20);
	gtk_fixed_put(GTK_FIXED(panel),config_window.desktopbg_cb, 10, 30);
	gtk_widget_show (config_window.desktopbg_cb);

	//shading
	shade_sc_label = gtk_label_new("shade");
	gtk_fixed_put(GTK_FIXED(panel),shade_sc_label, 10, 65);
	gtk_widget_show (shade_sc_label);
	adj = (GtkAdjustment *) gtk_adjustment_new (0, 0.0, 100.0, 1.0, 5.0, 0.0);
	config_window.shade_sc = gtk_hscale_new (GTK_ADJUSTMENT (adj));
	gtk_widget_set_usize(config_window.shade_sc, 100, 30);
	gtk_range_set_update_policy (GTK_RANGE (config_window.shade_sc), GTK_UPDATE_CONTINUOUS);
	gtk_scale_set_digits (GTK_SCALE(config_window.shade_sc), 0);
	gtk_scale_set_value_pos (GTK_SCALE(config_window.shade_sc), GTK_POS_TOP);
	gtk_scale_set_draw_value (GTK_SCALE(config_window.shade_sc), TRUE);
	gtk_fixed_put(GTK_FIXED(panel),config_window.shade_sc, 50, 50);
	gtk_widget_show (config_window.shade_sc);

	//backgdFiles stuff
	backgdFiles_b = gtk_button_new_with_label("bg files...");
	gtk_widget_show (backgdFiles_b);
	gtk_widget_set_usize(backgdFiles_b, 75, 20);
	gtk_fixed_put(GTK_FIXED(panel),backgdFiles_b, 10, 90);
	gtk_signal_connect(GTK_OBJECT(backgdFiles_b),"clicked",GTK_SIGNAL_FUNC(backgdFiles_b_pressed),GTK_OBJECT(backgdFiles_b));

	//backgd color button
	backgdColor_b = gtk_button_new_with_label("color...");
	gtk_widget_show (backgdColor_b);
	gtk_widget_set_usize(backgdColor_b, 50, 20);
	gtk_fixed_put(GTK_FIXED(panel),backgdColor_b, 10, 120);
	gtk_signal_connect(GTK_OBJECT(backgdColor_b),"clicked",GTK_SIGNAL_FUNC(backgdColor_b_pressed),GTK_OBJECT(backgdColor_b));

	return(panel);
}

//TEXT PANEL________________________________________________________________
GtkWidget* create_text_panel(void) {
GtkWidget *panel;
GdkBitmap *mask;
GdkPixmap *pixmap;
GtkStyle *style;
GtkWidget *back;
GtkWidget *textColor_b;
GtkWidget *font_b;
GtkWidget *text_sb;
GtkWidget *item;
GtkWidget *mylist;
int index;
char buf[8];

	panel = gtk_fixed_new();
	gtk_widget_show (panel);

	style = gtk_widget_get_style( mainwindow );
	pixmap = gdk_pixmap_create_from_xpm_d(mainwindow->window, &mask, &style->bg[GTK_STATE_NORMAL],text_xpm);
	back = gtk_pixmap_new (pixmap, mask);
	gtk_widget_set_usize(back, 460, 200);
	gtk_fixed_put(GTK_FIXED(panel),back, 0, 0);
	gtk_widget_show (back);

	//text Color text field
	config_window.textColor_tf = gtk_entry_new_with_max_length(128);
	gtk_widget_set_usize(config_window.textColor_tf, 70, 20);
	gtk_fixed_put(GTK_FIXED(panel),config_window.textColor_tf, 10, 40);
	gtk_widget_show (config_window.textColor_tf);

	//textColor button
	textColor_b = gtk_button_new_with_label("color...");
	gtk_widget_show (textColor_b);
	gtk_widget_set_usize(textColor_b, 50, 20);
	gtk_fixed_put(GTK_FIXED(panel),textColor_b, 10, 10);
	gtk_signal_connect(GTK_OBJECT(textColor_b),"clicked",GTK_SIGNAL_FUNC(textColor_b_pressed),GTK_OBJECT(textColor_b));

	//font stuff
	font_b = gtk_button_new_with_label("font...");
	gtk_widget_show (font_b);
	gtk_widget_set_usize(font_b, 75, 20);
	gtk_fixed_put(GTK_FIXED(panel),font_b, 130, 10);
	gtk_signal_connect(GTK_OBJECT(font_b),"clicked",GTK_SIGNAL_FUNC(font_b_pressed),GTK_OBJECT(font_b));

	//font label
	config_window.font_label = gtk_label_new("");
	//gtk_widget_set_usize(font_label, 70, 20);
	gtk_fixed_put(GTK_FIXED(panel),config_window.font_label, 130, 40);
	gtk_widget_show (config_window.font_label);

	//paddings
	//xPadding
	item = gtk_label_new("x pad");
	gtk_widget_show (item);
	gtk_fixed_put(GTK_FIXED(panel),item, 10, 70);
	index=1;
	config_window.xPadding_combo = gtk_combo_new();
	for (int i=8;i<=100;i+=index) {
		if (i>=20) index=5;
		if (i>=50) index=10;
		mylist = gtk_list_item_new();
		gtk_widget_show (mylist);
		sprintf(buf,"%d",i);
		item = gtk_label_new(buf);
		gtk_widget_show (item);
		gtk_container_add (GTK_CONTAINER (mylist), item);
		gtk_container_add (GTK_CONTAINER (GTK_COMBO (config_window.xPadding_combo)->list), mylist);
		}

	gtk_widget_set_usize(config_window.xPadding_combo, 60, 20);
	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(config_window.xPadding_combo)->entry),FALSE);
	gtk_fixed_put(GTK_FIXED(panel),config_window.xPadding_combo, 50, 70);
	gtk_widget_show (config_window.xPadding_combo);
	//yPadding
	item = gtk_label_new("y pad");
	gtk_widget_show (item);
	gtk_fixed_put(GTK_FIXED(panel),item, 10, 100);
	index=1;
	config_window.yPadding_combo = gtk_combo_new();
	for (int i=8;i<=100;i+=index) {
		if (i>=20) index=5;
		if (i>=50) index=10;
		mylist = gtk_list_item_new();
		gtk_widget_show (mylist);
		sprintf(buf,"%d",i);
		item = gtk_label_new(buf);
		gtk_widget_show (item);
		gtk_container_add (GTK_CONTAINER (mylist), item);
		gtk_container_add (GTK_CONTAINER (GTK_COMBO (config_window.yPadding_combo)->list), mylist);
		}

	gtk_widget_set_usize(config_window.yPadding_combo, 60, 20);
	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(config_window.yPadding_combo)->entry),FALSE);
	gtk_fixed_put(GTK_FIXED(panel),config_window.yPadding_combo, 50, 100);
	gtk_widget_show (config_window.yPadding_combo);

	//text.................
	config_window.text = gtk_text_new(NULL,NULL);
	gtk_widget_show (config_window.text);
	gtk_text_set_editable (GTK_TEXT (config_window.text), TRUE);
	gtk_widget_set_usize(config_window.text, 270, 80);
	gtk_fixed_put(GTK_FIXED(panel),config_window.text, 130, 70);
	text_sb = gtk_vscrollbar_new (GTK_TEXT (config_window.text)->vadj);
	gtk_widget_show (text_sb);
	gtk_widget_set_usize(text_sb, 15, 80);
	gtk_fixed_put(GTK_FIXED(panel),text_sb, 400, 70);


	return(panel);
}

//when quitting________________________________________________________________
void hide_config(void) {
	gtk_widget_hide (mainwindow);
	gtk_main_quit();
}

//display config window________________________________________________________
void Display_Config() {
	gtk_widget_show (mainwindow);
#ifdef DEBUG
printf("show conf win\n");
#endif
}

//set config window fields_____________________________________________________
void Set_Config_Fields() {
char buf[8];

	//configuring a Note...
	if (note_to_configure!=NULL) {
		//title
		gtk_entry_set_text (GTK_ENTRY(config_window.title_tf), note_to_configure->title);
		//fancy
		if (note_to_configure->border) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_window.border_cb),TRUE);
		else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_window.border_cb),FALSE);
		if (note_to_configure->sticky) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_window.sticky_cb),TRUE);
		else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_window.sticky_cb),FALSE);
		if (note_to_configure->ontop) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_window.ontop_cb),TRUE);
		else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_window.ontop_cb),FALSE);
		gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(config_window.angle_sc)), note_to_configure->angle);
		gtk_entry_set_text (GTK_ENTRY(config_window.alarm_tf), note_to_configure->alarm);
		gtk_entry_set_position(GTK_ENTRY(config_window.alarm_tf),0);
		//backgd
		gtk_entry_set_text (GTK_ENTRY(config_window.backgd_tf), note_to_configure->backgdColor);
		if (note_to_configure->transparent) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_window.transparent_cb),TRUE);
		else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_window.transparent_cb),FALSE);
		if (note_to_configure->desktopBackgd) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_window.desktopbg_cb),TRUE);
		else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_window.desktopbg_cb),FALSE);
		gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(config_window.shade_sc)), note_to_configure->shade);
		//text
		gtk_entry_set_text (GTK_ENTRY(config_window.textColor_tf), note_to_configure->textColor);
		gtk_label_set_text (GTK_LABEL(config_window.font_label), note_to_configure->fontName);
		sprintf(buf,"%d",note_to_configure->paddingX);
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(config_window.xPadding_combo)->entry),buf);
		sprintf(buf,"%d",note_to_configure->paddingY);
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(config_window.yPadding_combo)->entry),buf);
		//cleanup text area...
		gtk_text_set_point(GTK_TEXT(config_window.text),gtk_text_get_length( GTK_TEXT(config_window.text)));
		gtk_text_backward_delete(GTK_TEXT(config_window.text),gtk_text_get_length( GTK_TEXT(config_window.text)));
		gtk_text_insert (GTK_TEXT (config_window.text), NULL, NULL, NULL, note_to_configure->text, -1);
		}
}

//Update Note properties_________________________________________________________
void flush_data(void) {
char *buf;

#ifdef DEBUG
	printf("Flushing data...\n");
#endif
	if (note_to_configure!=NULL) {
		sprintf(note_to_configure->title,gtk_entry_get_text(GTK_ENTRY(config_window.title_tf)));
		//fancy
		note_to_configure->sticky=GTK_TOGGLE_BUTTON(config_window.sticky_cb)->active;
		note_to_configure->ontop=GTK_TOGGLE_BUTTON(config_window.ontop_cb)->active;
		note_to_configure->border=GTK_TOGGLE_BUTTON(config_window.border_cb)->active;
		note_to_configure->angle=(int)(gtk_range_get_adjustment(GTK_RANGE(config_window.angle_sc))->value);
		sprintf(note_to_configure->alarm,gtk_entry_get_text(GTK_ENTRY(config_window.alarm_tf)));
		//backgd
		note_to_configure->transparent=GTK_TOGGLE_BUTTON(config_window.transparent_cb)->active;
		note_to_configure->desktopBackgd=GTK_TOGGLE_BUTTON(config_window.desktopbg_cb)->active;
		note_to_configure->shade=(int)(gtk_range_get_adjustment(GTK_RANGE(config_window.shade_sc))->value);
		sprintf(note_to_configure->backgdColor,gtk_entry_get_text(GTK_ENTRY(config_window.backgd_tf)));
		//text
		sprintf(note_to_configure->textColor,gtk_entry_get_text(GTK_ENTRY(config_window.textColor_tf)));
		gtk_label_get(GTK_LABEL(config_window.font_label),&buf);
		sprintf(note_to_configure->fontName,buf);
		note_to_configure->paddingX=atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(config_window.xPadding_combo)->entry)));
		note_to_configure->paddingY=atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(config_window.yPadding_combo)->entry)));
		sprintf(note_to_configure->text,gtk_editable_get_chars( GTK_EDITABLE(config_window.text), 0, gtk_text_get_length( GTK_TEXT(config_window.text))));
	save_config();
	}
}

//when pressing OK in backgdFiles dialog ____________________________________
void select_backgdFile_ok( GtkWidget *w, GtkFileSelection *fs ) {
	gtk_entry_set_text (GTK_ENTRY (config_window.backgd_tf), gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
}

//when pressing backgdFiles button___________________________________________
void backgdFiles_b_pressed( GtkWidget *widget, GdkEvent  *event, gpointer   data ) {
GtkWidget *select_backgdFile_window;

	// Create a new file selection widget
	select_backgdFile_window = gtk_file_selection_new ("Backgd File selection");
	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (select_backgdFile_window)->ok_button), "clicked", (GtkSignalFunc) select_backgdFile_ok, select_backgdFile_window );
    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (select_backgdFile_window)->ok_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (select_backgdFile_window));
/* Connect the cancel_button to destroy the widget */
    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (select_backgdFile_window)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (select_backgdFile_window));

    gtk_widget_show(select_backgdFile_window);
}

//when pressing backgdColor button__________________________________________
void backgdColor_b_pressed(GtkWidget *widget) {
GtkWidget *backgdColorseldlg=NULL;
GtkWidget *colorsel;
GtkWidget *color_ok;
GtkWidget *color_cancel;

	backgdColorseldlg = gtk_color_selection_dialog_new("Select background color");
	colorsel = GTK_COLOR_SELECTION_DIALOG(backgdColorseldlg)->colorsel;
	gtk_signal_connect(GTK_OBJECT(colorsel), "color_changed", (GtkSignalFunc)backgdColor_changed, GTK_OBJECT(colorsel));
	color_ok = GTK_COLOR_SELECTION_DIALOG(backgdColorseldlg)->ok_button;
	gtk_signal_connect(GTK_OBJECT(color_ok), "clicked", (GtkSignalFunc)finished_with_backgdColor_cb, GTK_OBJECT(backgdColorseldlg));
	color_cancel = GTK_COLOR_SELECTION_DIALOG(backgdColorseldlg)->cancel_button;
	gtk_signal_connect(GTK_OBJECT(color_cancel), "clicked", (GtkSignalFunc)finished_with_backgdColor_cb, GTK_OBJECT(backgdColorseldlg));
	gtk_widget_show(backgdColorseldlg);
}

//backgd color DLG OK or CANCEL button pressed______________________________
void finished_with_backgdColor_cb( GtkWidget *widget, GtkColorSelection *colorsel ) {
	gtk_widget_hide(GTK_WIDGET(colorsel));
}

//backgd color DLG changed_________________________________________________
void backgdColor_changed( GtkColorSelection *colorsel ) {
gdouble color[4];
char buf[11];

	gtk_color_selection_get_color (colorsel,color);

	sprintf(buf,"%d/%d/%d",(int)(color[0]*255),(int)(color[1]*255),(int)(color[2]*255));
	gtk_entry_set_text (GTK_ENTRY (config_window.backgd_tf), buf);
}

//when pressing textColor button__________________________________________
void textColor_b_pressed(GtkWidget *widget) {
GtkWidget *textColorseldlg=NULL;
GtkWidget *colorsel;
GtkWidget *color_ok;
GtkWidget *color_cancel;

	textColorseldlg = gtk_color_selection_dialog_new("Select text color");
	colorsel = GTK_COLOR_SELECTION_DIALOG(textColorseldlg)->colorsel;
	gtk_signal_connect(GTK_OBJECT(colorsel), "color_changed", (GtkSignalFunc)textColor_changed, GTK_OBJECT(colorsel));
	color_ok = GTK_COLOR_SELECTION_DIALOG(textColorseldlg)->ok_button;
	gtk_signal_connect(GTK_OBJECT(color_ok), "clicked", (GtkSignalFunc)finished_with_textColor_cb, GTK_OBJECT(textColorseldlg));
	color_cancel = GTK_COLOR_SELECTION_DIALOG(textColorseldlg)->cancel_button;
	gtk_signal_connect(GTK_OBJECT(color_cancel), "clicked", (GtkSignalFunc)finished_with_textColor_cb, GTK_OBJECT(textColorseldlg));
	gtk_widget_show(textColorseldlg);
}

//text color DLG OK or CANCEL button pressed______________________________
void finished_with_textColor_cb( GtkWidget *widget, GtkColorSelection *colorsel ) {
	gtk_widget_hide(GTK_WIDGET(colorsel));
}

//backgd color DLG changed_________________________________________________
void textColor_changed( GtkColorSelection *colorsel ) {
gdouble color[4];
char buf[11];

	gtk_color_selection_get_color (colorsel,color);

	sprintf(buf,"%d/%d/%d",(int)(color[0]*255),(int)(color[1]*255),(int)(color[2]*255));
	gtk_entry_set_text (GTK_ENTRY (config_window.textColor_tf), buf);
}

//when pressing OK in font dialog ____________________________________
void select_font_ok( GtkWidget *w, GtkFontSelectionDialog *fs ) {
	gtk_label_set_text (GTK_LABEL(config_window.font_label), gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(fs)));
}

//when pressing font button___________________________________________
void font_b_pressed( GtkWidget *widget, GdkEvent  *event, gpointer   data ) {
GtkWidget *select_font_window;
char *buf;

	// Create a new file selection widget
	select_font_window = gtk_font_selection_dialog_new("Font selection");
	gtk_label_get(GTK_LABEL(config_window.font_label),&buf);
	gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(select_font_window),buf);
	gtk_signal_connect (GTK_OBJECT (GTK_FONT_SELECTION_DIALOG (select_font_window)->ok_button), "clicked", (GtkSignalFunc) select_font_ok, select_font_window );
    gtk_signal_connect_object (GTK_OBJECT (GTK_FONT_SELECTION_DIALOG (select_font_window)->ok_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (select_font_window));
/* Connect the cancel_button to destroy the widget */
    gtk_signal_connect_object (GTK_OBJECT (GTK_FONT_SELECTION_DIALOG (select_font_window)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (select_font_window));

    gtk_widget_show(select_font_window);
}

//----------------------------------------------------------------------------

/*
 * timer function...
 * ___________________________________________________________________
*/
/*
int timer_function( int fd, struct timeval *tv ) {
fd_set read_set;

	FD_ZERO( &read_set );
	FD_SET( fd, &read_set );
	switch( select( fd + 1, NULL, NULL, NULL, tv ) ) {
		case -1: return -1; break;
		case 0: return 0; break;
		}
	return 1;
}
*/


