#include "Notes.h"
#include <string>

CNotes::CNotes(void){
Display *disp;

	disp  = XOpenDisplay(NULL);
	id=Imlib_init(disp);
	cmap    = DefaultColormap(id->x.disp, id->x.screen);
	note_win = XCreateSimpleWindow(id->x.disp,  id->x.root, 0, 0, 300, 200, 0, 0, None);
	XSelectInput(id->x.disp, note_win, SubstructureNotifyMask | PropertyChangeMask | ExposureMask | ButtonPressMask | ButtonReleaseMask);
	if (!note_win) printf("Couldn't open window!\n");

   XMapWindow(id->x.disp, note_win);

	note_width = 300;
	note_height = 200;
}

CNotes::~CNotes(void) {
	XDestroyWindow(id->x.disp,note_win);
	XFreeColormap(id->x.disp,cmap);
	Imlib_free_pixmap(id,note_pixmap);
	Imlib_free_colors(id);
	XUnloadFont(id->x.disp,font);
	free(id);
}

/*
 * CONVERT RGB TO PIXEL VALUE OF PIXMAP
 */
void CNotes::convert_rgb_to_pixel(int r, int g, int b, unsigned long *pix) {
XVisualInfo *vis_info, vTemplate;
XColor col;
int count;

	col.red = r*256;
	col.green = g*256;
	col.blue = b*256;
	vTemplate.screen = id->x.screen;
   //vTemplate.class = TrueColor;
   vTemplate.depth = id->x.depth;
	vis_info = XGetVisualInfo(id->x.disp,VisualDepthMask | VisualScreenMask | VisualClassMask , &vTemplate, &count);
	XAllocColor(id->x.disp,cmap,&col);
	*pix = col.pixel;

	XFree(vis_info);
}

/*
 * REFRESH THE PIXMAP OF THE NOTE AT GIVE POSITION AND SIZE
 */
void CNotes::Refresh_Note(int x, int y, int w, int h) {
GC gc;
XGCValues xgcv;

#ifdef DEBUG
printf("refreshing at %d,%d / %dx%d\n",x,y,w,h);
#endif

	gc = XCreateGC(id->x.disp, note_win, 0, &xgcv);
	XCopyArea(id->x.disp,note_pixmap,note_win,gc,x,y,w,h,x,y);
	XFreeGC(id->x.disp,gc);
}

/*
 * UPDATE THE BACKGD IF DESKTOP BACKGD ON
 */
void CNotes::Watch_Window_Move(void) {
XWindowAttributes attrib;
int w_x, w_y;
Window child;

	XGetWindowAttributes(id->x.disp,note_win , &attrib);
	XTranslateCoordinates(id->x.disp, note_win, id->x.root, attrib.x, attrib.y, &w_x, &w_y, &child);
	if (w_x!=note_x || w_y!=note_y) {
		XSetWindowBackgroundPixmap(id->x.disp,note_win,Setup_DesktopBackgd());
		note_x = w_x;
		note_y = w_y;
		move_flag = 1;
		}
	else if (move_flag) {
		Build_Note();
		//XClearArea(id->x.disp, note_win, 0, 0, note_width, note_height, True);
		move_flag=0;
		}
}

/*
 * GET THE PIXMAP FROM THE ROOT WINDOW
 */
Pixmap CNotes::Setup_DesktopBackgd(void) {
unsigned int nb_children;
Window parent, *children;

#ifdef DEBUG
printf("Setting up desktopBackgd...\n");
#endif
	//XSync(id->x.disp,False);
	parent=note_win;
	for(int i=0;i<3;i++) {
		XQueryTree(id->x.disp,parent,&id->x.root,&parent,&children,&nb_children);
		if (parent!=id->x.root)
			XSetWindowBackgroundPixmap(id->x.disp,parent,ParentRelative);
		}
	XClearWindow(id->x.disp,note_win);
	XFree(children);
	//XSetWindowBackgroundPixmap(id->x.disp,note_win,ParentRelative);
	//XClearWindow(id->x.disp,note_win);

#ifdef DEBUG
printf("DesktopBackgd has been set up\n\n");
#endif
	return(ParentRelative);
}

/*
 * BUILD THE BACKGD OF THE NOTE : IMAGE OR PLAIN
 */
Pixmap CNotes::Setup_Background(void) {
int w_bg, h_bg, r, g, b;
Pixmap pm;
ImlibImage *im_bg;
unsigned long pix_color;
GC gc;
XGCValues xgcv;

#ifdef DEBUG
printf("Setting up backgdColor...\n");
#endif

	pm = XCreatePixmap(id->x.disp,note_win,note_width,note_height,id->x.depth);

	//check for image file...
	if (strstr(backgdColor,".")!=NULL) {
#ifdef DEBUG
printf("using background : %s\n",backgdColor);
#endif
		//load the backgd file...
		im_bg = Imlib_load_image(id,backgdColor);
		w_bg = im_bg->rgb_width;
		h_bg = im_bg->rgb_height;
		//tile the backgd file on the pixmap...
		for (int i=0;i<note_width;i+=w_bg)
			for (int j=0;j<note_height;j+=h_bg)
				Imlib_paste_image(id,im_bg,pm,i,j,w_bg,h_bg);
		Imlib_kill_image(id,im_bg);
		}
	else {
		sscanf(backgdColor,"%d/%d/%d",&r,&g,&b);
#ifdef DEBUG
printf("backgdColor = %d,%d,%d\n",r,g,b);
#endif
		gc = XCreateGC(id->x.disp, pm, 0, &xgcv);
		convert_rgb_to_pixel(r,g,b,&pix_color);
		XSetForeground(id->x.disp, gc, pix_color);
		XFillRectangle(id->x.disp,pm,gc,0,0,note_width,note_height);
		XFreeGC(id->x.disp,gc);
		}

#ifdef DEBUG
printf("BackgdColor has been set up\n\n");
#endif
	return(pm);
}

/*
 * CALCULATE TEXT EXTENDS TO GET NOTE WxH
*/
void CNotes::Setup_Note_Size(void) {
XCharStruct char_struct;
int y=0,asc,desc,direct, max_w=0, max_h=0;
char *token, *str_save=strdup(text);

#ifdef DEBUG
printf("Setting up Note size...\n");
#endif
	token = strtok(str_save,"\n");
	while(token!=NULL) {
		XQueryTextExtents(id->x.disp,font, token, strlen(token), &direct, &asc, &desc, &char_struct);
		char_asc = asc;
		char_height = asc+desc;
		if (char_struct.width>max_w) max_w = char_struct.width;
		if ((asc+desc+y)>max_h) max_h = y+char_height;
		token = strtok(NULL,"\n");
		y+=char_height;
		}
	note_width = max_w + 2*xPadding;
	note_height = max_h + 2*yPadding;
	free(token);
	free(str_save);
#ifdef DEBUG
printf("Note size has been set up...");
printf("note => %dx%d\n\n",note_width,note_height);
#endif

}

/*
 * DRAW THE TEXT OF NOTE ON A PIXMAP...
*/
Pixmap CNotes::Setup_Text(void) {
Pixmap pm;
GC gc;
XGCValues xgcv;
unsigned long pix_color;
int y=char_asc,r,g,b;
char *token, *str_save=strdup(text);

#ifdef DEBUG
printf("Setting up Note text...\n");
#endif
	pm = XCreatePixmap(id->x.disp,note_win,note_width,note_height,id->x.depth);
	gc = XCreateGC(id->x.disp, pm, 0, &xgcv);

	XSetForeground(id->x.disp, gc, 0);
	XFillRectangle(id->x.disp,pm,gc,0,0,note_width,note_height);
	//Set text color
	sscanf(textColor,"%d/%d/%d",&r,&g,&b);
	convert_rgb_to_pixel(r,g,b,&pix_color);
	XSetFont(id->x.disp,gc,font);
	XSetForeground(id->x.disp, gc, pix_color);
	//Actually draw the text on pixmap
	token = strtok(str_save,"\n");
	while(token!=NULL) {
		XDrawString(id->x.disp, pm, gc, xPadding+0, yPadding+y, token, strlen(token));
		token = strtok(NULL,"\n");
		y+=char_height;
		}

	free(token);
	free(str_save);
	XFreeGC(id->x.disp,gc);
#ifdef DEBUG
printf("Note text has been set up\n\n");
#endif

	return(pm);
}

/////////////////////////////////////////////
int CNotes::Blur_Color(int a,int b,int c, int d, int e, int f, int g, int h, int i) {
int k1,k2,k3,k4,k5,k6,k7,k8,k9;
k1=1; k2=1; k3=1;
k4=1; k5=5; k6=1;
k7=1; k8=1; k9=1;

return((a*k1+b*k2+c*k3+d*k4+e*k5+f*k6+g*k7+h*k8+i*k9)/(k1+k2+k3+k4+k5+k6+k7+k8+k9));
}

/////////////////////////////////////////////
Pixmap CNotes::Paste_Text(Pixmap bg_pm, ImlibImage *im_text) {
Pixmap mask=None;
ImlibImage *im_bg,*im_bg_aa;

#ifdef DEBUG
printf("Pasting text image on backgd pixmap\n");
#endif
	//put bg pixmap in image
	Imlib_paste_image(id,im_text,bg_pm,0,0,note_width,note_height);
	im_bg = Imlib_create_image_from_drawable(id,bg_pm,mask,0,0,note_width,note_height);
	im_bg_aa = Imlib_create_image_from_drawable(id,bg_pm,mask,0,0,note_width,note_height);
	for (int i=3*note_width;i<(note_width-1)*note_height*3;i+=3) {
		if (im_text->rgb_data[i]||im_text->rgb_data[i+1]||im_text->rgb_data[i+2]) {
			//red
			im_bg_aa->rgb_data[i] = Blur_Color(im_bg->rgb_data[i-3*note_width-3],im_bg->rgb_data[i-3*note_width],im_bg->rgb_data[i-3*note_width+3],im_bg->rgb_data[i-3],im_bg->rgb_data[i],im_bg->rgb_data[i+3],im_bg->rgb_data[i+3*note_width-3],im_bg->rgb_data[i+3*note_width],im_bg->rgb_data[i+3*note_width+3]);
			//green
			im_bg_aa->rgb_data[i+1] = Blur_Color(im_bg->rgb_data[i-3*note_width-2],im_bg->rgb_data[i-3*note_width+1],im_bg->rgb_data[i-3*note_width+4],im_bg->rgb_data[i-2],im_bg->rgb_data[i+1],im_bg->rgb_data[i+4],im_bg->rgb_data[i+3*note_width-2],im_bg->rgb_data[i+3*note_width+1],im_bg->rgb_data[i+3*note_width+4]);
			//blue
			im_bg_aa->rgb_data[i+2] = Blur_Color(im_bg->rgb_data[i-3*note_width-1],im_bg->rgb_data[i-3*note_width+2],im_bg->rgb_data[i-3*note_width+5],im_bg->rgb_data[i-1],im_bg->rgb_data[i+2],im_bg->rgb_data[i+5],im_bg->rgb_data[i+3*note_width-1],im_bg->rgb_data[i+3*note_width+2],im_bg->rgb_data[i+3*note_width+5]);
			}
		}

	Imlib_render(id,im_bg_aa,note_width,note_height);
	bg_pm = Imlib_move_image(id,im_bg_aa);
	Imlib_apply_image(id,im_bg_aa,note_win);

	Imlib_kill_image(id,im_bg_aa);
	Imlib_kill_image(id,im_bg);

#ifdef DEBUG
printf("final Antialias pixmap has been created.\n\n");
#endif
	return(bg_pm);
}

/*
 * BUILD THE NOTE : BACKGD + TEXT
*/
void CNotes::Build_Note(void) {
Pixmap text_pm, mask=None;
ImlibColorModifier color_modif;
ImlibImage *im_bg,*im_text;
ImlibColor color;
GC gc;
XGCValues xgcv;
	if (note_pixmap) {
		XFreePixmap(id->x.disp,note_pixmap);
		}

#ifdef DEBUG
printf("building note...\n");
#endif

	//Load the desired font
	font = XLoadFont(id->x.disp,fontName);
	if (!font) printf("Couldn't load font\n");

	//Calculate note extends
	Setup_Note_Size();
	XResizeWindow(id->x.disp,note_win,note_width,note_height);

	//BACKGD STUFF
	if (!transparent) {
		if (desktopBackgd) {
			note_pixmap = XCreatePixmap(id->x.disp,note_win,note_width,note_height,id->x.depth);
			Setup_DesktopBackgd();
			XSetWindowBackgroundPixmap(id->x.disp,note_win,ParentRelative);
			XClearWindow(id->x.disp,note_win);
			gc = XCreateGC(id->x.disp, note_pixmap, 0, &xgcv);
			XCopyArea(id->x.disp,note_win,note_pixmap,gc,0,0,note_width,note_height,0,0);
			XFreeGC(id->x.disp,gc);
			if (!note_pixmap) {note_pixmap = Setup_Background();}
			}
		else note_pixmap = Setup_Background();
		//Put bg pixmap in imlib image to apply a shading...
		if (shade) {
			im_bg = Imlib_create_image_from_drawable(id,note_pixmap,mask,0,0,note_width,note_height);
			//Defines the color modifiers
			Imlib_get_image_modifier(id,im_bg,&color_modif);
			color_modif.brightness=256-shade*256/100;
			Imlib_set_image_modifier(id,im_bg,&color_modif);
			Imlib_render(id,im_bg,note_width,note_height);
			note_pixmap = Imlib_move_image(id,im_bg);
			Imlib_kill_image(id,im_bg);
			}
		}
	//TEXT STUFF
	text_pm = Setup_Text();
	//Put pixmap in im_text and set transparent color to black
	im_text = Imlib_create_image_from_drawable(id,text_pm,mask,0,0,note_width,note_height);
	color.r=0; color.g=0; color.b=0;
	Imlib_set_image_shape(id,im_text,&color);
	if (!transparent) {
		note_pixmap = Paste_Text(note_pixmap,im_text);
		}
	else {
		Imlib_apply_image(id,im_text,note_win);
		note_pixmap = text_pm;
		}

	Imlib_kill_image(id,im_text);
	XFreePixmap(id->x.disp,text_pm);


#ifdef DEBUG
printf("note built\n\n");
#endif
}


/*
 * Put note ontop or not....
*/
void CNotes::Setup_Layer(int ot) {
//I got it from gkrellm: thanks to Bill Wilson...who got it from xmms who got it from gnomeui-lib... ;)

#define	WIN_LAYER_NORMAL	4
#define	WIN_LAYER_ONTOP		6
XEvent xev;
int layer;

	layer = ot ? WIN_LAYER_ONTOP : WIN_LAYER_NORMAL;

	xev.type = ClientMessage;
	xev.xclient.type = ClientMessage;
	xev.xclient.window = note_win;
	xev.xclient.message_type = XInternAtom(id->x.disp, "_WIN_LAYER", False);
	xev.xclient.format = 32;
	xev.xclient.data.l[0] = layer;
	xev.xclient.data.l[1] = CurrentTime;
	XSendEvent(id->x.disp, id->x.root, True, SubstructureNotifyMask, (XEvent *) &xev);
}

/*
 * Setup the note sticky or not
*/
void CNotes::Setup_Sticky(int stick)
{
//for masks, cf gnome.c in enlightenment sources...
//I got it from gnomeui-lib... ;)
XEvent xev;
int state;
#define  WIN_STATE_STICKY           (1<<0) /* everyone knows sticky */
#define  WIN_STATE_MINIMIZED        (1<<1) /* ??? */
#define  WIN_STATE_MAXIMIZED_VERT   (1<<2) /* window in maximized V state */
#define  WIN_STATE_MAXIMIZED_HORIZ  (1<<3) /* window in maximized H state */
#define  WIN_STATE_HIDDEN           (1<<4) /* not on taskbar but window visible */
#define  WIN_STATE_SHADED           (1<<5) /* shaded (NeXT style) */
#define  WIN_STATE_HID_WORKSPACE    (1<<6) /* not on current desktop */
#define  WIN_STATE_HID_TRANSIENT    (1<<7) /* owner of transient is hidden */
#define  WIN_STATE_FIXED_POSITION   (1<<8) /* window is fixed in position even */
#define  WIN_STATE_ARRANGE_IGNORE   (1<<9)  /* ignore for auto arranging */

	state = 0;
  
	if (stick) state = WIN_STATE_STICKY;
	else state = 0;

	xev.type = ClientMessage;
	xev.xclient.type = ClientMessage;
	xev.xclient.window = note_win;
	xev.xclient.message_type = XInternAtom(id->x.disp, "_WIN_STATE", False);
	xev.xclient.format = 32;
	xev.xclient.data.l[0] = (long)(WIN_STATE_STICKY |
				       WIN_STATE_MAXIMIZED_VERT |
				       WIN_STATE_MAXIMIZED_HORIZ |
				       WIN_STATE_HIDDEN |
				       WIN_STATE_SHADED |
				       WIN_STATE_HID_WORKSPACE |
				       WIN_STATE_HID_TRANSIENT |
				       WIN_STATE_FIXED_POSITION |
				       WIN_STATE_ARRANGE_IGNORE);
	xev.xclient.data.l[1] = (long)state;
	xev.xclient.data.l[2] = CurrentTime;
	XSendEvent(id->x.disp, id->x.root, True, SubstructureNotifyMask, (XEvent *) &xev);
}

/*
 * Setup the note border
*/
void CNotes::Setup_Border(int bord) {
static Atom atom = 0;
int data[5];
XEvent ev;
//this un/mapping piss me off but I couldn't find any better...
//I think even Enlightenment do the same.
	XUnmapWindow(id->x.disp, note_win);
	ev.type = MapRequest;

	atom = XInternAtom(id->x.disp, "_MOTIF_WM_HINTS", False);
	data[0] = 1L << 1;
	data[1] = 0x0;
	data[2] = bord;
	data[3] = 0x0;
	data[4] = 0x0;
	XChangeProperty(id->x.disp, note_win, atom, atom, 32, PropModeReplace, (unsigned char*)&data, 5);
	XMapWindow(id->x.disp, note_win);
}

/*
 * ______________________________________________
 *
 * ---------------SET FUNCTIONS------------------
 * ______________________________________________
 */
void CNotes::Set_Border(int b) {
	if (b!=border) Setup_Border(b);
	border = b;
#ifdef DEBUG
printf("border = %d\n",border);
#endif
}

void CNotes::Set_Sticky(int s) {
	if (s!=sticky) Setup_Sticky(s);
	sticky = s;
#ifdef DEBUG
printf("sticky = %d\n",sticky);
#endif
}

void CNotes::Set_Ontop(int o) {
	if (ontop!=o) Setup_Layer(o);
	ontop = o;
#ifdef DEBUG
printf("ontop = %d\n",ontop);
#endif
}

void CNotes::Set_Angle(int a) {
	angle = (double)a*3.14159/180.0;
#ifdef DEBUG
printf("angle = %f\n",angle);
#endif
}

void CNotes::Set_DesktopBackgd(int d) {
	desktopBackgd = d;
#ifdef DEBUG
printf("DesktopBackgd = %d\n",desktopBackgd);
#endif
}

void CNotes::Set_Shade(int sh) {
	shade = sh;
#ifdef DEBUG
printf("shade = %d\n",shade);
#endif
}

void CNotes::Set_XPadding(int xp) {
	xPadding = xp;
#ifdef DEBUG
printf("xPadding = %d\n",xPadding);
#endif
}

void CNotes::Set_YPadding(int yp) {
	yPadding = yp;
#ifdef DEBUG
printf("yPadding = %d\n",yPadding);
#endif
}

void CNotes::Set_Transparent(int tr) {
	transparent = tr;
#ifdef DEBUG
printf("transparent = %d\n",transparent);
#endif
}

void CNotes::Set_Alarm(char *a) {
	sprintf(alarm,a);
#ifdef DEBUG
printf("Alarm = %s\n",alarm);
#endif
}

void CNotes::Set_BackgdColor(char *s) {
	sprintf(backgdColor,s);
#ifdef DEBUG
printf("backgdColor = %s\n",backgdColor);
#endif
}

void CNotes::Set_TextColor(char* s) {
	sprintf(textColor,s);
#ifdef DEBUG
printf("textColor = %s\n",textColor);
#endif
}

void CNotes::Set_Title(char* t) {
	sprintf(title,t);
	XStoreName(id->x.disp, note_win, title);
#ifdef DEBUG
printf("title = %s\n",title);
#endif
}

void CNotes::Set_FontName(char *s) {
	sprintf(fontName,s);
#ifdef DEBUG
printf("fontName = %s\n",fontName);
#endif
}

void CNotes::Set_Text(char *s) {
	sprintf(text,s);
#ifdef DEBUG
printf("Text = %s\n",text);
#endif
}

void CNotes::Set_Position(int x, int y) {
	XMoveWindow(id->x.disp,note_win,x,y);
#ifdef DEBUG
printf("Window moved at %d.%d\n",x,y);
#endif
}


/*
 * ______________________________________________
 *
 * ---------------GET FUNCTIONS------------------
 * ______________________________________________
 */

Display *CNotes::Get_Display(void) const {
return(id->x.disp);
}

int CNotes::Get_Border(void) const {
return(border);
}

int CNotes::Get_Sticky(void) const {
return(sticky);
}

int CNotes::Get_Ontop(void) const {
return(ontop);
}

int CNotes::Get_DesktopBackgd(void) const {
return(desktopBackgd);
}

int CNotes::Get_Angle(void) const {
return((int)(angle/3.1415*180.0));
}

int CNotes::Get_Transparent(void) const {
return(transparent);
}

int CNotes::Get_Shade(void) const {
return(shade);
}

int CNotes::Get_Width(void) const {
return(note_width);
}

int CNotes::Get_Height(void) const {
return(note_height);
}

int CNotes::Get_XPadding(void) const {
return(xPadding);
}

int CNotes::Get_YPadding(void) const {
return(yPadding);
}

const char* CNotes::Get_Alarm(void) const {
return(alarm);
}

const char* CNotes::Get_BackgdColor(void) const {
return(backgdColor);
}

const char* CNotes::Get_TextColor(void) const {
return(textColor);
}

const char* CNotes::Get_Title(void) const {
return(title);
}

const char* CNotes::Get_FontName(void) const {
return(fontName);
}

const char* CNotes::Get_Text(void) const {
return(text);
}

const char* CNotes::Get_XMLText(void) const {
   const std::string XMLescapes = "&<>'\"";
   const std::string XMLescaped[] = 
      {
         "&amp;",
         "&lt",
         "&gt",
         "&quot",
         "&apos"
      };
   
     // & first so iterative replacement is made easier :-)
   std::string xmlified(text);
   int i = 0; 
   bool specCharFound = false;
   for (std::string::const_iterator specChar = XMLescapes.begin();
        specChar != XMLescapes.end();specChar++)
   {
        // if this char found
      unsigned int quot = xmlified.find(*specChar);
      if (quot != std::string::npos)
      {
         specCharFound = true;
      }

   };
   std::string copyXML;
   if (specCharFound)
   {
      std::string::const_iterator strPos = xmlified.begin();
        
      for(;strPos != xmlified.end();++strPos)
        {
            switch (*strPos)
            {
                case '>':
                   copyXML += XMLescaped[2];
                    break;
                case '<':
                   copyXML += XMLescaped[1];
                    break;
                case '"':
                   copyXML += XMLescaped[3];
                    break;
                case '\'':
                   copyXML += XMLescaped[4];
                    break;
                default:
                    copyXML += *strPos;
            }
        }
      return copyXML.c_str();
   }

   return(text);
}

