#include <util/PlatformUtils.hpp>
#include <util/TransService.hpp>
#include <sax2/SAX2XMLReader.hpp>
#include <sax2/XMLReaderFactory.hpp>
#include "NotesConfig.h"
#include "SAX2Handlers.h"
#include <iostream>
#include <fstream>
#include <string>

CNotesConfig::CNotesConfig() {
	notesList = NULL;
	defaultNote = NULL;
}

CNotesConfig::~CNotesConfig(void) {
	XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::Terminate();
}

void CNotesConfig::read_config(char *filename) {
    XERCES_CPP_NAMESPACE_QUALIFIER SAX2XMLReader* parser=NULL;

	try {
		XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::Initialize();
		parser = XERCES_CPP_NAMESPACE_QUALIFIER XMLReaderFactory::createXMLReader();
		}

	catch (const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch) {
		printf("Error during initialization! %s\n",XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(toCatch.getMessage()));
		}

	try {
		SAX2Handlers handler("iso-8859-1", XERCES_CPP_NAMESPACE_QUALIFIER XMLFormatter::UnRep_CharRef,false);
		if (parser) {
			//n = (NoteStruct*)malloc(sizeof(NoteStruct));
			//notesList = (LinkedList*)malloc(sizeof(LinkedList));
			parser->setContentHandler(&handler);
			parser->setErrorHandler(&handler);
			//handler.setNotesList(notesList);
			parser->parse(filename);
			handler.getNotesList(&notesList);
			handler.getDefaultNote(&defaultNote);
			}
		}

	catch (const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch) {
		printf("An error occured\n  Error:%s\n",XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(toCatch.getMessage()));
		}

	if (!defaultNote) {
		defaultNote = (NoteStruct*)malloc(sizeof(NoteStruct));
		fillDefaultNote(defaultNote);
		}

}

void CNotesConfig::fillDefaultNote(NoteStruct *n) {
	n->border=1;
	n->angle=0;
	n->sticky=0;
	n->ontop=0;
	sprintf(n->alarm,"at 00:00 on 00/00/0000 every 0 days");
	n->transparent=0;
	n->desktopBackgd=0;
	n->shade=10;
	n->paddingX=10;
	n->paddingY=5;
	sprintf(n->fontName,"-misc-fixed-medium-r-normal-*-20-*-*-*-c-*-iso8859-1");
	sprintf(n->textColor,"128/128/0");
	sprintf(n->backgdColor,"255/255/0");
	sprintf(n->title,"note title");
	sprintf(n->text,"note text");
}

void CNotesConfig::getDefaultNote(NoteStruct **n) {
	*n = defaultNote;
}

void CNotesConfig::setDefaultNote(NoteStruct *n) {
	defaultNote = n;
}

void CNotesConfig::getNotesList(LinkedList **l) {
	*l = notesList;
}

void CNotesConfig::setNotesList(LinkedList *l) {
	notesList = l;
}

void escapeStringForXML(std::ostream& stream, const std::string& strValue)
{
    std::string::const_iterator strPos = strValue.begin();
    
    for(;strPos != strValue.end();++strPos)
    {
        switch (*strPos)
        {
            case '>':
                stream << "&gt;";
                break;
            case '<':
                stream << "&lt;";
                break;
            case '"':
                stream << "&quot;";
                break;
            case '\'':
                stream << "&apos;";
                break;
            case '&':
                stream << "&amp;";
                break;
            default:
                stream << *strPos;
        }
    }
}

int CNotesConfig::save_config(char *filename) {
    LinkedList *list;
    CNotes *n;
    std::ofstream f(filename);
	if (f) {
		f << "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n\n";
		f << "<ENotes>\n";
		write_default_note(f);
		for (list=notesList;list;list=list->tail) {
			n = (CNotes*)list->head;
		
			f << "\t<note border=\"" << n->Get_Border() << "\" angle=\"" 
              << n->Get_Angle() << "\" sticky=\"" << n->Get_Sticky() << "\" ontop=\""
              << n->Get_Ontop() << "\" transparent=\"" << n->Get_Transparent() << "\" shade=\"" << n->Get_Shade() 
              << "\" desktopBackgd=\"" << n->Get_DesktopBackgd() << "\" paddingX=\""
              << n->Get_XPadding() << "\" paddingY=\"" << n->Get_YPadding() << "\">\n";
			f << "\t\t<alarm>" << n->Get_Alarm() << "</alarm>\n";
			f << "\t\t<fontName>" << n->Get_FontName() << "</fontName>\n";
			f << "\t\t<textColor>" << n->Get_TextColor() << "</textColor>\n";
			f << "\t\t<backgdColor>" << n->Get_BackgdColor() << "</backgdColor>\n";
			f << "\t\t<title>" << n->Get_Title() << "</title>\n";
			f << "\t\t<text>" << n->Get_Text() << "</text>\n";
			f << "\t</note>\n";
// 			fprintf(f,"\t<note border=\"%d\" angle=\"%d\" sticky=\"%d\" ontop=\"%d\" transparent=\"%d\" shade=\"%d\" desktopBackgd=\"%d\" paddingX=\"%d\" paddingY=\"%d\">\n",n->Get_Border(),n->Get_Angle(),n->Get_Sticky(),n->Get_Ontop(),n->Get_Transparent(),n->Get_Shade(),n->Get_DesktopBackgd(),n->Get_XPadding(),n->Get_YPadding());
// 			fprintf(f,"\t\t<alarm>%s</alarm>\n",n->Get_Alarm());
// 			fprintf(f,"\t\t<fontName>%s</fontName>\n",n->Get_FontName());
// 			fprintf(f,"\t\t<textColor>%s</textColor>\n",n->Get_TextColor());
// 			fprintf(f,"\t\t<backgdColor>%s</backgdColor>\n",n->Get_BackgdColor());
// 			fprintf(f,"\t\t<title>%s</title>\n",n->Get_Title());
// 			fprintf(f,"\t\t<text>%s</text>\n",n->Get_Text());
// 			fprintf(f,"\t</note>\n");
        };

        f << "</ENotes>\n";
        f.close();
		return 0;
		}
	else {
		printf("Could not openfile [%s] for writing: notes cannot be saved!\n",filename);
		return -1;
		}
}

void CNotesConfig::write_default_note(std::ostream& f) {
// 	f.form("\t<defaultnote border=\"%d\" angle=\"%d\" sticky=\"%d\" ontop=\"%d\" transparent=\"%d\" shade=\"%d\" desktopBackgd=\"%d\" paddingX=\"%d\" paddingY=\"%d\">\n",defaultNote->border,defaultNote->angle,defaultNote->sticky,defaultNote->ontop,defaultNote->transparent,defaultNote->shade,defaultNote->desktopBackgd,defaultNote->paddingX,defaultNote->paddingY);
    const NoteStruct* n =  defaultNote;
    f << "\t<defaultnote border=\"" << n->border << "\" angle=\"" 
      << n->angle << "\" sticky=\"" << n->sticky << "\" ontop=\""
      << n->ontop << "\" transparent=\"" << n->transparent << "\" shade=\"" << n->shade
      << "\" desktopBackgd=\"" << n->desktopBackgd << "\" paddingX=\""
      << n->paddingX << "\" paddingY=\"" << n->paddingY << "\">\n";
	f << "\t\t<alarm>" << defaultNote->alarm << "</alarm>\n";
	f << "\t\t<fontName>" << defaultNote->fontName << "</fontName>\n";
	f << "\t\t<textColor>" << defaultNote->textColor << "</textColor>\n";
	f << "\t\t<backgdColor>" << defaultNote->backgdColor << "</backgdColor>\n";
	f << "\t\t<title>" << defaultNote->title << "</title>\n";
	f << "\t\t<text>" << defaultNote->text << "</text>\n";
	f << "\t</defaultnote>\n";
}
